/*
 * Author: Roman Rakus <rak0018@vsb.cz>
 */

#include <GL/glut.h>

#include <math.h>
#include <sys/time.h>

static int width = 800;
static int height = 600;

static float angle = 0.0;
static float dt = 0.0;

static struct timeval last_idle_time;

// distance between vertices in triangles used for table
#define STEP 0.25

// The size of table from -size to +size -> size * 2
#define TABLE_SIZE 10.0

GLUquadric* qobj;

void InitGraphics(void)
{
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glShadeModel(GL_SMOOTH);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);
  glEnable(GL_COLOR_MATERIAL);

  qobj = gluNewQuadric();
  gluQuadricNormals(qobj, GLU_SMOOTH);
}

void rotateLight(float angle)
{
  static int direction = 1;
  static float langle = 0;
  
  langle += direction * dt * 12.3;
  if (langle > 30.0 || langle < -30.0)
    direction *= -1;

  glRotatef(langle, 1, 0, 0);
}

void renderDesk(float size)
{
  glNormal3f(0.0, 1.0, 0.0);
  float x = -size, z = -size;
  glPushMatrix();
    glBegin(GL_TRIANGLE_STRIP);
      do
        {
          for(z = -size; z <= size; z += STEP)
            {
              glVertex3f(x+STEP, 0.0, z);
              glVertex3f(x, 0.0, z);
            }
          x+=STEP; if (x>=size) break;
          glVertex3f(x, 0.0, size);
          for(z = size; z >= -size; z -= STEP)
            {
              glVertex3f(x+STEP, 0.0, z);
              glVertex3f(x, 0.0, z);
            }
          glVertex3f(x, 0.0, -size);
        } while (x < size);
    glEnd();
  glPopMatrix();
}


void renderLeg(float size)
{
  gluCylinder(qobj, size/15.0, size/15.0, size/2.0, size*20, size/2.0);
}

void renderTable(void)
{
  glColor3f(0.545, 0.27, 0.075);
  renderDesk(TABLE_SIZE);

  glPushMatrix();
    glTranslatef(TABLE_SIZE*9/10, 0.0, TABLE_SIZE*9/10);
    glRotatef(90, 1.0, 0.0, 0.0);
    renderLeg(TABLE_SIZE);
  glPopMatrix();
  glPushMatrix();
    glTranslatef(-TABLE_SIZE*9/10, 0.0, TABLE_SIZE*9/10);
    glRotatef(90, 1.0, 0.0, 0.0);
    renderLeg(TABLE_SIZE);
  glPopMatrix();
  glPushMatrix();
    glTranslatef(-TABLE_SIZE*9/10, 0.0, -TABLE_SIZE*9/10);
    glRotatef(90, 1.0, 0.0, 0.0);
    renderLeg(TABLE_SIZE);
  glPopMatrix();
  glPushMatrix();
    glTranslatef(TABLE_SIZE*9/10, 0.0, -TABLE_SIZE*9/10);
    glRotatef(90, 1.0, 0.0, 0.0);
    renderLeg(TABLE_SIZE);
  glPopMatrix();
}

void renderAxis(void)
{
  glDisable(GL_LIGHTING);
  glBegin(GL_LINES);
    glColor3f(1.0, 0.0, 0.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(15.0, 0.0, 0.0);
    
    glColor3f(0.0, 1.0, 0.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 15.0, 0.0);

    glColor3f(0.0, 0.0, 1.0);
    glVertex3f(0.0, 0.0, 0.0);
    glVertex3f(0.0, 0.0, 15.0);
  glEnd(); 
  glEnable(GL_LIGHTING);
}

void renderObjects(void)
{
  float light_direction[3] = {0.0, -1.0, 0.0};
  glMatrixMode(GL_MODELVIEW);
//  renderAxis();
  renderTable();
  glColor3f(0.1, 0.8, 0.3);
  glPushMatrix();
    glRotatef(angle*8.1, 0, 1, 0);
    glTranslatef(0.0, 2.0, 0.0);
    glutSolidTeapot(2);
  glPopMatrix();
  glPushMatrix();
    rotateLight(angle);
    glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light_direction);
  glPopMatrix();
}

void display(void)
{
  float light0_pos[4] = {0.0, 10.0, 0.0, 1.0};
  float light0_exp[1] = {66.0};
  float light0_cutoff[1] = {44.0};

  float light1_pos[4] = {1.0, 1.0, 1.0, 0.0};
  float light1_dif[4] = {0.2, 0.4, 0.4, 1.0};
  float light1_spec[4] = {0.1, 0.2, 0.2, 1.0};
 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glLoadIdentity();
  gluLookAt(20.0 * cos(angle/2), 10.0, -20.0 * sin(angle/2),
            0.0, 0.0, -1.0, 0.0, 1.0, 0.0);
  glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);
  glLightfv(GL_LIGHT0, GL_SPOT_EXPONENT, light0_exp);
  glLightfv(GL_LIGHT0, GL_SPOT_CUTOFF, light0_cutoff);

  glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_dif);
  glLightfv(GL_LIGHT1, GL_SPECULAR, light1_spec);


  renderObjects();
  glFlush();
  glutSwapBuffers();
}

void reshape(GLint w, GLint h)
{
   width = w;
   height = h;

   glViewport(0, 0, width, height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(65.0, (float)width / height, 1, 1000);
   glMatrixMode(GL_MODELVIEW);
}

void idle(void)
{
  struct timeval time_now;
  gettimeofday(&time_now, NULL);
  dt = (float)(time_now.tv_sec  - last_idle_time.tv_sec) +
  1.0e-6*(time_now.tv_usec - last_idle_time.tv_usec);

  angle += dt;

  last_idle_time = time_now;
  glutPostRedisplay();
}

int main(int argc, char** argv)
{
  glutInit(&argc, argv);
  glutInitWindowSize(width, height);
  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  glutCreateWindow("Roman Rakus projekt PG1");

  InitGraphics();  

  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutIdleFunc(idle);
  gettimeofday(&last_idle_time, NULL);
  glutMainLoop();
  gluDeleteQuadric(qobj);
  return 0;
}
